/*
	iSet.r

	Created: November 13, 2005 19:12

	used for creating listings for instruction sets
*/

type 'iSLs'
{
	literal longint = 'isls';
	literal longint = 'dlk';

	integer;			/* architecture version */

	integer = $$CountOf(iSetCount);
	array iSetCount
	{
		integer;		/* resource ID */
		wide array MachCount
		{
			pstring;
		};
		byte = 0;
	};
};

#define iSet_User_Types			\
	integer_displacement = 0,	\	/* can offset anything (masm doesnt care) */
								\	/*  -- values are disassembled in decimal */
			PC_near		 = 2,	\	/* current PC address plus value */
			PC_plus_next = 3		/* adds to PC after alignment size */

type 'iSet'
{
	literal longint = 'iset';
	literal longint		version_zero = 'dck';

	string[4];			/* hex setup (0x%h,(60)=0x60h) */
	string[4];			/* literal setup (#i%*,(23)=#i23*) */

 /* flags */
	boolean		multiform, uniform;		/* uniform->sum of widths must not overflow */
	boolean		preferNoLit, preferLit;	/* prefer to show the literal symbol */
	boolean = 0;						/* reserved */
	boolean = 0;						/* reserved */
	boolean		noPredec, hasPredec;	/* has pre-decrementing addressing mode */
	boolean		noPreinc, hasPreinc;	/* has pre-incrementing addressing mode */
	boolean		noPstdec, hasPstdec;	/* has post-decrementing addressing mode */
	boolean		noPstinc, hasPstinc;	/* has post-decrementing addressing mode */

 bAlignSize:
	integer bitstring[4]; /* alignment byte size -- zero means unaligned (otherwise */
				 		  /*  all unidentified bits are ignored until next alignment) */
	integer bitstring[4]	hexadecimal= 0;	/* prefered base number */

 /* the opcode search offsets are mainly for the disassembler */
	byte;				/* opcode search first offset */
	byte;				/* opcode search second offset */

 bProcCount:
	integer = $$CountOf(ProcCount);
	array ProcCount
	{
		pstring;
		byte;			/* category */
		boolean		noFPU, hasFPU;			/* has built-in FPU */
		boolean		noCoproc, canCoproc;	/* can interface with coprocessors */
		fill bitstring[7];
	};
	byte = 0;

	integer = $$CountOf(EffectiveAddressCount);
	array EffectiveAddressCount
	{	/* (values as of version zero) */
		longint			/* type */
			iSet_User_Types;

		wide array CustomCount
		{
			pstring;
		};
		byte = 0;
	};

	unsigned longint = $$CountOf(EntryCount);
	wide array EntryCount
	{

	#define OpWidth ($$Byte(bOpcodeWidth[$$ArrayIndex(EntryCount)])>>4)

		pstring;
		hex string[($$Word(bProcCount)+7)&-8];
		array OpcodeCount
		{
		bOpcodeWidth:
			unsigned bitstring[4];		  /* width (in bits) */
			switch
			{
				case Constant:
					/* a specific value that defines the opcode */
					key bitstring[4] = 0;
					//unsigned integer;
					integer bitstring[(!OpWidth)*16 + ((OpWidth+7)&24)];
					align word;
				case LgConstant:
					key bitstring[4] = 1;
					hex string[OpWidth];  /* width is in bytes */

				case User_Type:
					key bitstring[4] = 2;
					byte iSet_User_Types; /* the User list defines representation */
				case LgUser_Type:
					key bitstring[4] = 3;
					byte iSet_User_Types; /* width is in bytes */
				case Macro_Type:
					key bitstring[4] = 4;
					byte;				  /* points to an entry in the Macro list */
				case Register_Type:
					key bitstring[4] = 5;
					byte;				  /* points to an entry in the Register list */
				case FRegister_Type:
					key bitstring[4] = 6;
					/* addressing a register in a field (Rn:Rn) */
					byte;				  /* operand index (zero-based) */
					byte;				  /* field register */

				case Exchange:
					key bitstring[4] = 7;
					/* exchange two operands (ordering) */
					byte;				  /* operand index A (zero-based) */
					byte;				  /* operand index B (zero-based) */
				case Mnemonic:
					key bitstring[4] = 8;
					pstring;			  /* instruction name */

				case Matching_Type:
					key bitstring[4] = 9;
					/* (operand type is same as previous operand) */
				case Ignored:
					key bitstring[4] = 10;
					/* these bits are ignored */

				case Matching:
					key bitstring[4] = 11;
					byte;				  /* reproduces an operand index (zero-based) */

				case LgLiteral:
					/** width denotes byte size (after alignment size, if any) **/
					key bitstring[4] = 12;
				case Nonzero_Literal:
					/* zero calls for a hidden one
					/*  (the value after the highest unsigned value) */
					key bitstring[4] = 13;
				case Signed_Literal:
					key bitstring[4] = 14;
				case Unsigned_Literal:
					key bitstring[4] = 15;
			};
		};
		byte = 0;
	};
	byte = 0;
};
